  
"
    __________________________________________________
    @Alejandro F. Reimondo 19/11/97
    This proyect sources are property of : Alejandro F. Reimondo
    Its use is NOT legal without written rights.
    Amenabar 3476 Piso: 2 Dto: B.
    Ciudad de Buenos Aires.
    Argentina.
    Tel: (54)-1-703-5586.
    __________________________________________________
"!

ClassHierarchyBrowser subclass: #OSClassHierarchyBrowser
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: 
    ' CharacterConstants VirtualKeyConstants '  !

Inspector subclass: #OSInspector
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: 
    ' ColorConstants OperatingSystemConstants ' !


!OSClassHierarchyBrowser methods !
   
canDrop: anObject into: aPane
    "Private - True if the receiver can handle anObject now...
    Implemented by Ale for drag&drop support"

    ^anObject isBehavior
    or: [
        anObject isCompiledMethod
        "   and: [ anObject classField isClass ]   "
    ]!

copyToClipboard
    "Private - Write the source for the selected class
        in chunk file format to the clipboard..
    Implemented by Ale..."
    | stream |
    selectedClass isNil ifTrue: [^self].
    stream := WriteStream on: String new.
    CursorManager execute change.
    selectedClass fileOutOn: stream.
    stream nextChunkPut: String new.
    (ClassReader forClass: selectedClass class) fileOutOn: stream.
    (ClassReader forClass: selectedClass) fileOutOn: stream.
    CursorManager normal change.
    Clipboard setString: stream contents.!

disableClassesMenu
    "Private - Disable the classes menu.
     Touched by Ale for more functionality"
    self classesMenu
        disableItem: #removeSubClass;
        disableItem: #references;
        disableItem: #fileOutClass;
        disableItem: #fileOutClassWithSubclasses;
        disableItem: #hideShow;
        disableItem: #browse;
        disableItem: #addSubClass;
        disableItem: #removeSources;
        disableItem: #printOut;
        disableItem: #printOutWithSubclasses;
        disableItem: #copyToClipboard!
 
drop: anObject into: aPane
    "Private - Set the current class & method with anObject
    Implemented by Ale for drag&drop support"

    anObject isBehavior
    ifTrue: [ self findClass: anObject symbol ].

    anObject isCompiledMethod
    ifTrue: [
        self findClass: anObject classField symbol.
        anObject classField isClass
        ifTrue: [self instanceButton: nil ]
        ifFalse: [self classButton: nil ].
        (self paneAt: #selectors:) restoreSelected: anObject selector.
        self changed: #text:.
    ].

    (self paneAt: #text:) setFocus!

enableClassesMenu
    "Private - Enable the classes menu.
     Touched by Ale for more functionality"
    self classesMenu
        enableItem: #removeSubClass;
        enableItem: #references;
        enableItem: #fileOutClass;
        enableItem: #fileOutClassWithSubclasses;
        enableItem: #hideShow;
        enableItem: #browse;
        enableItem: #addSubClass;
        enableItem: #removeSources;
        enableItem: #printOut;
        enableItem: #printOutWithSubclasses;
        enableItem: #copyToClipboard!
  
endPrinting: device
    "Private - Write the remaining contents
    in the output device to the printer.
    Implemented By Ale"
    device contents size > 0
    ifTrue: [device contents outputToPrinter]!
   
fileNameFor: aClass
    "Private - Returns an apropiate file name for aClass.
    Implemented By Ale"
        ^(aClass respondsTo: #abreviation)
        ifTrue: [aClass abreviation]
        ifFalse: [aClass name]!
  
fileOutClass
        "Private - Write the source for the selected class
         in chunk file format to a file named with the class
         name reduced to 8 characters, extension 'cls'."
    | class file fileName fileDialog |
    ( class := self selectedClass ) isNil ifTrue: [ ^nil ].
    fileName := File fileName: (self fileNameFor: selectedClass) extension: 'cls'.
    fileDialog := FileDialog new
        saveTitle: 'File out ', class name, ' class'
        fileName: fileName.
    ( file := fileDialog file ) isNil ifTrue: [ ^nil ].

    CursorManager execute changeFor: [
        (ClassReader forClass: class) fileOut: file
    ].!
 
fileOutClassWithSubclasses
        "Private - Write the source for the selected class
         in chunk file format to a file named with the class
         name reduced to 8 characters, extension st."
    | class file fileName fileDialog |
    ( class := self selectedClass ) isNil ifTrue: [ ^nil ].
    fileName := File fileName: (self fileNameFor: selectedClass) extension: 'st'.
    fileDialog := FileDialog new
        saveTitle: 'File out ', class name, ' class'
        fileName: fileName.
    ( file := fileDialog file ) isNil ifTrue: [ ^nil ].
    ( ClassReader forClass: class ) fileOutAll: file!

findClass: defaultName
        "Private - prompt for the name of an existing class, using defaultName
        as the initial name value; answer the name of the matching class, or
        nil if no match found."

    ^self findClass: defaultName prompt: true!
 
findClass: defaultName prompt: aBoolean
        "Private - prompt for the name of an existing class, using defaultName
        as the initial name value; answer the name of the matching class, or
        nil if no match found."
    | name pattern matches pt simil match |
    aBoolean | defaultName isNil
    ifTrue: [
        name := Prompter
            title: 'Find Class'
            prompt: 'Enter a class name:'
            default: defaultName.
        name isNil ifTrue: [ ^nil ]
    ] ifFalse: [ name := defaultName ].

    #addedByOStyle.
    simil := Array with: 0 with: nil.
    Smalltalk keysDo: [:key|
        match := key matchOf: name.
        match = 1 ifTrue: [
            (Smalltalk at: key asSymbol) isClass
            ifFalse: [ ^(Smalltalk at: key asSymbol) class symbol ].
            ^key
        ].
        ((Smalltalk at: key) isClass and: [ match > simil first ])
        ifTrue: [
            simil
                at: 1 put: match;
                at: 2 put: key
        ]
    ].
    (Smalltalk at: simil last ifAbsent: []) isBehavior ifTrue: [ ^simil last ].

    "exact match"
    ( ( Smalltalk includesKey: name asSymbol ) and: [ ( Smalltalk at: name asSymbol ) isClass ] )
        ifTrue: [ ^name ].

    "matches beginning of a class"
    pattern := Pattern new: name asUpperCase.
    matches := OrderedCollection new.
    Object allClasses do: [ :class |
        pt := pattern match: class name index: 1 each: [ :e | e asUpperCase ].
        ( pt notNil and: [ pt x = 1 ] )
            ifTrue: [ matches add: class name ] ].
    matches size > 1 ifTrue: [
        matches := matches reject: [ :s | s size > name size ] ].

    "if no match found, or too many matches found, forget it"
    matches size ~= 1 ifTrue: [ MessageBox message: 'Class not found'. ^nil ].
    ^matches first!

instVarSetUp: selected
        "Private - An instance variable was selected as
         visible/modifiable by outside.
        Implemented By Ale"
    | result aClass compileBlock
		initialization initializeSelector |
    aClass := instanceSelectedLast
        ifTrue: [selectedClass]
        ifFalse: [selectedClass class].
    (aClass instVarNames includes: selected)
    ifFalse: [Terminal bell.^nil].
    compileBlock := [:aString|
    result := self compile: aString
        notifying: Transcript pane
        in: aClass.
    result isNil
        ifTrue: [
            MessageBox
                message: ('Compiler Error compiling :\',aString) withCrs
        ]
        ifFalse: [
            SourceManager current
                logSource: aString
                forSelector: result key
                inClass: aClass]
     ].

	initialization := Notifier isControlKeyDown
		ifTrue: [
			initializeSelector := 'initialize'
				,(String with: selected first asUpperCase)
				,(selected copyFrom: 2 to: selected size).
	'

	',selected,' isNil ifTrue: [ self ',initializeSelector,' ].']
		ifFalse: [ String new ].

    compileBlock
        value: selected,'
    "Returns the instance var of the receiver
     for external manipulation"',initialization,'
    ^',selected;
        value: selected,': anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    ',selected,' := anObject'.

	initializeSelector notNil ifTrue: [
		compileBlock
			value: initializeSelector,'
	" Private - Initializes the receiver''s ',selected,'"

	',selected,' := self notImplementedYet.'
	].

    methodSelectedLast := false.
    selectedInstVar :=  nil.
    self disableMethodsMenu.
 "   self changed: #instanceVars:.  "
    self changed: #selectors:.
    self changed: #text:
!
   
label
        "Private - Answer the window label."
    ^'Super Class Hierarchy Browser'!
  
menu: hierarchyPane
	"Private - Set the hierarchy pane menu."

    | menu |
	super menu: hierarchyPane.
	menu := hierarchyPane menuAttribute.
    menu
		appendSeparator;
        "  appendItem: 'Remove \uSources' replaceEscapeCharacters selector: #removeSources; "
        appendItem: '\uPrint Out...' replaceEscapeCharacters selector: #printOut;
        appendItem: 'Pr\uint Out All...' replaceEscapeCharacters selector: #printOutWithSubclasses;
		appendSeparator;
        appendItem: '\uCopy to Clipboard' replaceEscapeCharacters selector: #copyToClipboard;
        yourself.!
  
openOn: aCollection

    super openOn: (aCollection asArray collect: [:each|
        each isBehavior ifTrue: [each] ifFalse: [Smalltalk at: each]]).

    (self paneAt: #instanceVars:)
            when: #doubleClicked: send: #instVarSetUp:
!
 
openOn: aCollection selecting: aClass

    self
        openOn: aCollection;
        selectedClass: aClass;
        yourself!
  
openSelecting: aClass

    self
        openOn: Smalltalk rootClasses;
        selectedClass: aClass;
        yourself!

printingDevice
        "Private - Returns the printer device for the receiver."
    ^(Smalltalk at: #PrinterStream
        ifAbsent: [^WriteStream on: (String new: 1000)]
        ) new!
   
printOut
        "Private - Write the source for the selected class
         in chunk file format to the printer."
    | printer |
    selectedClass isNil ifTrue: [^self].
    printer := self printingDevice.
    CursorManager execute change.
    (ClassReader forClass: selectedClass) fileOutClassOn: printer.
    printer nextPut: 12 asCharacter.
    self endPrinting: printer.
    CursorManager normal change!
 
printOutMethod
        "Private - Write the source for the selected method
         in chunk file format to the printer"
    | printer |
    selectedClass isNil ifTrue: [^self].
    selectedMethod isNil ifTrue: [^self].
    printer := self printingDevice.
    CursorManager execute change.
    (ClassReader forClass:
        (instanceSelectedLast
                ifTrue: [selectedClass]
                ifFalse: [selectedClass class]))
        fileOutMethod: selectedMethod on: printer.
    printer nextPut: 12 asCharacter.
    self endPrinting: printer.
    CursorManager normal change!
  
printOutWithSubclasses
        "Private - Write the source for the selected class
         as well as its subclasses in chunk file format to
         the printer."
    | printer |
    selectedClass isNil ifTrue: [^self].
    printer := self printingDevice.
    CursorManager execute change.
    selectedClass withAllSubclasses do: [:c |
        (ClassReader forClass: c) fileOutClassOn: printer.
        printer nextPut: 12 asCharacter
    ].
    self endPrinting: printer.
    CursorManager normal change.!

removeSelector
        "Private - Remove the selected method."
    | method aString theClass |
    methodSelectedLast ifFalse: [ ^nil ].
    selectedMethod isNil ifTrue: [ ^nil ].
    theClass := instanceSelectedLast
        ifTrue: [ selectedClass ]
        ifFalse: [ selectedClass class ].
    method := theClass compiledMethodAt: selectedMethod.
    method notNil ifTrue: [ aString := method source ].
    aString isNil ifFalse: [ Clipboard setString: aString ].
    ^super removeSelector
!
  
selectedClass: aClass
    "Private - Set the receiver for seen aClass."

    | className class supers blanks result |
    self textModified ifTrue: [^self].
    className :=
        aClass isClass
        ifTrue: [ aClass symbol ]
        ifFalse: [ aClass asExistingSymbolOrNil ].
    (className isNil or: [className isEmpty]) ifTrue: [ ^ self ].
    className := className asExistingSymbolOrNil.
    class := Smalltalk at: className ifAbsent: [^self].
    CursorManager execute changeFor: [
        supers := (Array with: class), class allSuperclasses.
        blanks := 0.
        1 to: supers size do: [ :i |
            (originalClasses includes: (supers at: i))
               ifTrue: [ blanks := i ]].
        blanks = 0 ifTrue: [
            ^MessageBox message: 'Class "', className,
                            '" not found in this browser.' ].
        supers do: [ :cl |
            hiddenClasses remove: cl ifAbsent: []].
        self update: originalClasses.
        methodSelectedLast := false.
        selectedMethod := nil.
        selectedInstVar := nil.
        selectedClass := class.
        self changed: #hierarchy:
            with: #restoreSelected:
            with: (((String new: ((blanks - 1 max: 0) * 2)) atAllPut: $ ),
                class name).
        self enableClassesMenu.
        self disableMethodsMenu.
        self
            changed: #instanceVars: ;
            changed: #selectors: ;
            changed: #text: ]! !



!OSInspector methods !

createView
        "Private - create the panes for the receiver window."

    | gap pane |
    gap := 3.

    super createView.

    self addSubpane: (
        (pane := StaticText new)
            owner: self;
             setName: #instanceClass:;
            framingBlock: [:box| box leftTop extent: box width @ ( gap + TextFont height) ];
            startGroup;
            when: #needsContents send: #instanceClass: to: self with: pane;
            font: TextFont;
            yourself );
        yourself.

    (self paneAt: #instVarList:)
        framingBlock: [:box| 0@ ( gap + TextFont height) rightBottom: box width // 3 @ box height ];
        yourself.

    instPane
        framingBlock: [:box| box width // 3 @ ( gap + TextFont height) rightBottom:  box rightBottom ];
        yourself.!
  
getInstance

    | i |
    instIndex isNil ifTrue:[^nil].

    ^instIndex = 1
        ifTrue:  [object]
        ifFalse: [
            object class isBytes
                ifTrue: [(object at: (instIndex - 1))  ]
                ifFalse: [(i := self getInstVarIndex) ~= 0 ifTrue: [
                    (object instVarAt: i)]]]!
  
inspectMenu: pane
        "Private - Set the dictionary
         inspector list pane menu."
    | menu |
    menu :=  Menu new
                title: '&Inspect';
                owner: self;
                appendItem: '&Inspect' selector: #inspectSelection;
                appendSeparator;
                appendItem: 'Inspect self all References' selector: #objectReferences;
                appendItem: 'Browse self class' selector: #objectBrowseClass;
                appendItem: 'Inspect self dependents' selector: #objectDependents;
                appendItem: 'Inspect self dependecies' selector: #objectDependencies;
                appendSeparator;
                appendItem: 'Inspect instance all References' selector: #instanceReferences;
                appendItem: 'Browse instance class' selector: #instanceBrowseClass;
                appendItem: 'Inspect instance dependents' selector: #instanceDependents;
                appendItem: 'Inspect instance dependecies' selector: #instanceDependencies.
    menu disableItem: #inspectSelection.
    pane setMenu: menu!

instanceBrowseClass

    "Callback for the menu item titled 'Browse class'.
     (Generated by WindowBuilder)"

    instIndex isNil ifTrue: [^self].
    ClassHierarchyBrowser new
        openSelecting: self getInstance class.!
   
instanceClass: aPane

    "Callback for the #getContents event in the StaticText named 'instanceClass'.
     (Generated by WindowBuilder)"
    | instance |
    aPane contents: 'instance var class: ',
        (instIndex isNil ifTrue:['']
        ifFalse: [
            instance := self getInstance.
            instance class name
        ])!
 
instanceDependencies

    "Callback for the menu item titled 'Inspect dependecies'.
     (Generated by WindowBuilder)"
    self notImplementedYet!
  
instanceDependents

    "Callback for the menu item titled 'Inspect all Dependents'.
     (Generated by WindowBuilder)"
    | allRef |
    instIndex isNil ifTrue: [^self].
    allRef := self getInstance allDependents.
    allRef size = 0 ifTrue: [^MessageBox message: 'No Dependents'].
    allRef size = 1 ifTrue: [^allRef first inspect].
    allRef inspect.!

instanceReferences

    "Callback for the menu item titled 'Inspect all References'.
     (Generated by WindowBuilder)"
    | allRef |
    instIndex isNil ifTrue: [^self].
    allRef := self getInstance allReferences.
    allRef size = 0 ifTrue: [^MessageBox message: 'No References'].
    allRef size = 1 ifTrue: [^allRef first inspect].
    allRef inspect.!

objectBrowseClass

    "Callback for the menu item titled 'Browse class'.
     (Generated by WindowBuilder)"

    ClassHierarchyBrowser new openSelecting: object class.!
  
objectDependencies

    "Callback for the menu item titled 'Inspect dependecies'.
     (Generated by WindowBuilder)"
    self notImplementedYet!

objectDependents

    "Callback for the menu item titled 'Inspect dependents'.
     (Generated by WindowBuilder)"
    | allRef |
    allRef := object allDependents.
    allRef size = 0 ifTrue: [^MessageBox message: 'No Dependents'].
    allRef size = 1 ifTrue: [^allRef first inspect].
    allRef inspect.!
  
objectReferences

    "Callback for the menu item titled 'Inspect all References'.
     (Generated by WindowBuilder)"
    | allRef |

    allRef := object allReferences.
    allRef size = 0 ifTrue: [^MessageBox message: 'No References'].
    allRef size = 1 ifTrue: [^allRef first inspect].
    allRef inspect.!

selectInstance: aListPane

    super selectInstance: aListPane.
    self changed: #instanceClass:.! !


! Behavior methods !
comment
    "Returns the receiver's comment."

    #addedByOStyle.
    ^self instanceClass comment!
comment: aString
    "Change the receiver's comment.
    Implemented by Ale..."

    #addedByOStyle.
    ^self instanceClass comment: aString! !

! GraphicInspector methods !
createView
        "Private - create the panes for the receiver window.
    Touched by Ale for mouse location report..."
    | pane |
    #modifiedByOStyle.
    self addSubpane:
        ( ( pane := GraphPane new )
            setName: #graphicInstance: ;
            when: #display send: #graphicInstance: to: self with: pane;
            when: #button1Down send: #setFocus to: pane;
            when: #button1Moved send: #mouseMoved: to: self with: pane;
            framingRatio:
                ( ( Rectangle leftTopUnit rightAndDown: ( 1/3 @ 0 ) )
                    extentFromLeftTop: 2/3 @ 1 ) ).
	self mainView when: #activated send: #update to: pane.
    super createView!
mouseMoved: aPane
    "Private - Display the mouse coordinates in the label."

    #addedByOStyle.
    self label: 'Inspecting: ', object class printString,' [',aPane mouseLocation asString,']'! !

! NotificationManager methods !
reinitDelayed
        "Private - initialize and open a new Transcript.
    Touched by Ale for transcript text saving..."

    | text | #modifiedByOStyle.
    (Smalltalk at: #Transcript ifAbsent: [ nil ]) notNil
    ifTrue: [ text := (Smalltalk at: #Transcript) contents ].

    HelpManager exit.
    SystemWindow exit.
    ( Smalltalk includesKey: #SmalltalkEmulator )
        ifTrue: [ ( Smalltalk at: #SmalltalkEmulator ) exit ].
    self closeAllWindows.
    self initialize.
    self initializeTranscript.
    ( Smalltalk includesKey: #SmalltalkEmulator )
        ifTrue: [ ( Smalltalk at: #SmalltalkEmulator ) startUp ].
    SystemWindow startUp.

    #modifiedByOStyle.
    text isNil ifFalse: [ (Smalltalk at: #Transcript) pane contents: text ].

    HelpManager startUp! !

! SourceManager methods !
compressChangesOf: aClass into: aStream
        "Private - Write latest version of all methods
         in aClass that are in the change log to aStream.
    Touched by Ale for shrinked format (redundant headers removed)"

    | method |
    #modifiedByOStyle.

    aClass selectors
        detect: [:each|
         (aClass compiledMethodAt: each) sourceIndex = 2 ]
        ifNone: [^self]. "no changed methods..."

    "Place 'aClass methods'  header"
    self putHeaderOf: aClass into: aStream.

    "Place methods sources in chunk format"
    aClass selectors asSortedCollection do: [ :s |
        method := aClass compiledMethodAt: s.
        method sourceIndex = 2
        ifTrue: [
                aStream cr.
                self
                    putMethod: method
                    withIndex: 2
                    to: aStream
        ]
    ].

    "place class group termination chunk"
    aStream nextPutAll: ' ' . "Changed for better fileIn..."!
compressChanges
        "Build a new change log file retaining
         only the latest version of changed
         methods in the current change log.
         Save the image to the image file.
    Touched by Ale for store of class definitions of current touched classes"

    | logDirectory stream dialog allClasses class |
    ( SessionModel testDiskSpaceRequiredToCompressChanges: BackupImage )
        ifFalse: [ ^self ].

    BackupImage ifTrue: [ self backupChanges ifFalse: [ ^self error: 'Error backing up change.log' ] ].
    dialog := ProgressIndicatorDialog new
        noCancel;
        open: WindowLabelPrefix message: 'Compressing Changes...'.
    logDirectory := ( Sources at: 2 ) file directory.
    stream := logDirectory newFile: 'change.tmp'.
    stream lineDelimiter: Cr.
    allClasses := self getSourceClasses.

    #modifiedByOStyle.
    "store class headers for changed classes"
    dialog message: 'Storing class headers...'; percent: 0.
    1 to: allClasses size do: [ :i |
        class := allClasses at: i.
        #platformDependent. "message #isInLibrary may be #isInDLL in previous versions..."
        (class isInLibrary not "not a Digitalk class..."
        or: [class instVarNames isInLibrary not]) "basic class but instances changed..."
        ifTrue: [self fileOut: class into: stream].
        dialog percent: (i / allClasses size * 100 ) truncated
    ].
    "store methods"
    dialog message: 'Storing methods...'; percent: 0.

    1 to: allClasses size do: [ :i |
        class := allClasses at: i.

        #modifiedByOStyle.
        dialog message: 'Storing ',class name,'...'.

        self compressChangesOf: class class into: stream.
        self compressChangesOf: class into: stream.
        dialog percent: ( i / allClasses size * 100 ) truncated ].
    stream close.
    ( Sources at: 2 ) close.
    File remove: ( Sources at: 2 ) pathName.
    File rename: stream pathName to: ( Sources at: 2 ) pathName.
    Sources at: 2 put: ( logDirectory file: ( Sources at: 2 ) file name ).
    ( Sources at: 2 ) lineDelimiter: Cr.
    dialog message: 'Saving Image File...';
        percent: 0;
        percent: 100;
        close.
    SessionModel current saveSession!
fileOut: aClass into: aStream
    "Place the header definition of aClass into aStream.
     Implemented by Ale 29/08/91 "
    | stream |
    #addedByOStyle.
    (aClass selectors detect: [:one|
            (aClass compiledMethodAt: one) sourceIndex = 2
         ] ifNone: []) isNil
     & (aClass class selectors detect: [:one|
            (aClass class compiledMethodAt: one) sourceIndex = 2
         ] ifNone: []) isNil
    ifTrue: [^nil]. "no changes has been made..."

    stream := WriteStream on: (String new: 200).
    aClass fileOutOn: stream.
    aStream
        cr; cr;
        nextPutAll: '"define class"';
        nextChunkPut: stream contents;
        flush! !

! Inspector class methods !
new
    "Returns an instance of OSInspector..."

    #addedByOStyle.
    self == Inspector
    ifTrue: [ ^OSInspector new ].

    ^super new! !

! Class methods !
comment
    "Returns the receiver's comment."

    | holder |
    #addedByOStyle.

    comment notNil ifTrue: [ ^comment ].

    holder := DefinitionInstaller current.
    (holder respondsTo: #commentForClass:)
    ifTrue: [ ^holder commentForClass: self ].

    ^comment!
comment: aString
    "Record the argument as the class comment of the reciever.
    Modified to reflect comment changes in the receiver if the Definition Instaler does not alter comment. Ale."

    DefinitionInstaller current setClassComment: aString forClass: self
    .#modifiedByOStyle.
    comment := aString.!
fileOutOn: aStream
    "Append the class definition message for the receiver to aStream.
    Touched by Ale for comment: support..."

    | aString |
    aStream cr;
        nextPutAll: self superclass printString; space;
        nextPutAll: self kindOfSubclass; space;
        nextPutAll: self symbol storeString; cr; space; space.
    self isBits
        ifFalse: [
            aStream nextPutAll: 'instanceVariableNames: '.
            (aString := self instanceVariableString) isEmpty
                ifFalse:[aStream cr; nextPutAll: '    '].
            aStream
                nextPutAll: aString storeString;
                cr; space; space].
    aStream
        nextPutAll: 'classVariableNames: '.
    (aString := self classVariableString) isEmpty
        ifFalse:[aStream cr; nextPutAll: '    '].
    aStream
        nextPutAll: aString storeString; cr; space; space;
        nextPutAll: 'poolDictionaries: '.
    (aString := self sharedVariableString) isEmpty
        ifFalse:[aStream cr; nextPutAll: '    '].
    aStream
        nextPutAll: aString storeString

    .#modifiedByOStyle.
    comment size > 0
    ifTrue: [
        aStream nextPutAll: '.';cr;cr;
            nextPutAll: '  (Smalltalk at: ',self symbol storeString
                ,') comment: ',comment storeString
    ].!
removeFromSystem: checkForInstances
        "Private - Remove the receiver from Smalltalk.  Report
         an error if there are any subclasses or
         instances of the receiver. If checkForInstances
         is true then we check if there are any instances of the
         receiver.
    Touched to remove class withAllSubclasses & automatic instance remotion."

    | index index2 allInstances |  #modifiedByOStyle.
    ((OrderedCollection new
        add: UndefinedObject;
        add: Class;
        add: True;
        add: False;
        add: DeletedClass;
        add: EmptySlot;
        add: SmallInteger;
        yourself)
            includes: self)
                ifTrue: [^self error: 'Class cannot be removed.'].

    checkForInstances ifTrue: [
        (allInstances := self allInstances) notEmpty
        ifTrue: [
            #modifiedByOStyle.
            (MessageBox confirm: self name,' has instances; do you want to eliminate them ?')
            ifFalse: [ ^self error: 'Has instances' ].
            allInstances do: [:anInstance| anInstance become: nil ].
        ]
     ].

    #modifiedByOStyle.
    self subclasses notEmpty ifTrue: [
        self subclasses do: [:each| each removeFromSystem: checkForInstances ]
    ].

    DefinitionInstaller current removeClass: self! !

! ViewManager methods !
compressChanges
    "Private - Compress the change.log file.
    Implemented By Ale"

    #addedByOStyle.
   SourceManager current compressChanges.!
cleanUp
    "Private - Clean Up of hole system windows.
    Creating a new transcript.
    Implemented By Ale"

    #addedByOStyle.
   "clean up windows"
   Notifier reinitialize!
unusedMemory
    "Private - Reports unusedMemory in bytes(perform GC).
    Implemented By Ale"

    #addedByOStyle.
   MessageBox message: (Smalltalk unusedMemory "* 512 wrong documentation in method comment") asString.!
sendersInText
    "Private - Show senders of selected text message.
    Selected text will be considered as:
        if only a word:
            first letter lowerCase -> message selector
            first letter uppercase -> Global var reference
        if many words: treated as an evaluable expresion
    Implemented By Ale"

    | activeTextPane text exit |
    #addedByOStyle.
    exit := [ Terminal bell. ^nil ].
    activeTextPane := self mainView activeTextPane.
    activeTextPane isNil ifTrue: [^self].
    text := activeTextPane selectedString.
    text size = 0 ifTrue: [ exit value ].
    text := text trimBlanks.
    text isEmpty ifTrue: [ exit value ].

    text asArrayOfSubstrings size > 1
    ifTrue: [ text := Compiler evaluate: text ]
    ifFalse: [ text := text asExistingSymbolOrNil ].
    text isNil ifTrue: [ exit value ].

    text isSymbol ifFalse: [
        text := Smalltalk keyAtValue: text ifAbsent: exit
    ].

    text first isUpperCase
    ifTrue: [ text := Smalltalk associationAt: text ifAbsent: exit ].

    Smalltalk sendersOf: text.!
implementorsInText
    "Private - Show implementors of selected text message.
    Selected text will be considered as:
        if only a word:
            first letter lowerCase -> message selector
            first letter uppercase -> Global var reference
        if many words: treated as an evaluable expresion
    Implemented By Ale"

    | activeTextPane text exit |
    #addedByOStyle.
    exit := [ Terminal bell. ^nil ].
    activeTextPane := self mainView activeTextPane.
    activeTextPane isNil ifTrue: [^self].
    text := activeTextPane selectedString.
    text size = 0 ifTrue: [ exit value ].
    text := text trimBlanks.
    text isEmpty ifTrue: [ exit value ].

    text asArrayOfSubstrings size > 1
    ifTrue: [ text := Compiler evaluate: text ]
    ifFalse: [ text := text asExistingSymbolOrNil ].
    text isNil ifTrue: [ exit value ].

    text isSymbol ifFalse: [
        text := Smalltalk keyAtValue: text ifAbsent: exit
    ].

    text first isUpperCase
    ifTrue: [ text := Smalltalk associationAt: text ifAbsent: exit ].

    Smalltalk implementorsOf: text.!
purgeResources
    "Private - Purge System resources.
    Implemented By Ale"

    #addedByOStyle.
    Bitmap purgeUnusedBitmaps.
    Icon purgeUnusedIcons.
    FileHandle cleanHandles.!
findClassInText
    "Private - Find class in text selected...
    Implemented By Ale"

    | activeTextPane text exit aClass |
    #addedByOStyle.
    exit := [ Terminal bell. ^nil ].

    activeTextPane := self mainView activeTextPane.
    activeTextPane isNil ifTrue: [^self].
    text := activeTextPane selectedString.
    text size = 0 ifTrue: [ exit value ].
    text := text trimBlanks.
    text isEmpty ifTrue: [ exit value ].
    text := text asStream nextWord.
    (self respondsTo: #findClass:prompt:)
    ifTrue: [ text := self findClass: text prompt: false ].

    text := text asExistingSymbolOrNil.
    text isNil ifTrue: [ exit value ].

    (self respondsTo: #selectedClass:)
    ifTrue: [ self selectedClass: text ]
    ifFalse: [
        aClass := Smalltalk at: text ifAbsent: [ exit value ].
        aClass isClass ifFalse: [ aClass := aClass class ].
        ClassHierarchyBrowser new openSelecting: aClass instanceClass
    ]! !

! MethodBrowser methods !
method: listPane
        "Private - Display source code for
         the selected method."
    selectedMethod := methods at: listPane selection.
    selectedMethod := selectedMethod classField
        compiledMethodAt: selectedMethod selector.
    selectedMethod isNil
        ifTrue: [self error: 'deleted method'].
    methods at: listPane selection put: selectedMethod.
    self changed: #text:.
    self enableMethodsMenu.
    positions isNil ifFalse: [
        positions isPoint
            ifTrue: [ methodPane showSelectionFrom: positions x to: positions y + 1 ]
            ifFalse: [
                methodPane
                    showSelectionFrom: positions position
                    to: positions endPosition ].
				methodPane forceSelectionOntoDisplay. #modifiedByOStyle.
	]! !

! ClassHierarchyBrowser class methods !
new
    "Returns an instance of OSClassHierarchyBrowser..."

    #addedByOStyle.
    self == ClassHierarchyBrowser
    ifTrue: [ ^OSClassHierarchyBrowser new ].

    ^super new! !

! SmalltalkWindowPolicy class methods !
addSmalltalkToolItemsTo: aMenu
        "Private - add menu items that launch development tools to aMenu.
    Touched by Ale for better menu construction..."

    aMenu
        appendItem: 'New \uWorkspace\tCtrl+W' replaceEscapeCharacters selector: #openWorkspace accelKey: $W accelBits: AfControl;
        appendItem: '\uBrowse Classes\tCtrl+B' replaceEscapeCharacters selector: #openClassBrowser accelKey: $B accelBits: AfControl;
        appendItem: 'Browse Dis\uk\tCtrl+K' replaceEscapeCharacters selector: #openDiskBrowser accelKey: $K accelBits: AfControl;
        appendItem: 'Browse Se\urvices' replaceEscapeCharacters selector: #openServiceManagerWindow.
    "
    self toolItems do: [ :item |
        aMenu appendItem: ( item at: 1 ) action: ( item at: 2 ) commandKey: ( item at: 3 ) ]
    "
    #modifiedByOStyle.
    self toolItems size = 0 ifTrue: [ self initializeToolItems ].
    self addAditionalSmalltalkToolItems: self toolItems to: aMenu!
addAditionalSmalltalkToolItems: tools to: aMenu
    "Private - Add aditional tool menu items that launch tool alternatives to aMenu.
    See method #initializeToolItems"

    | action commandKey label appendSubMenu subMenu |

    #addedByOStyle.
    tools size > 0 ifFalse: [ ^self ].

    appendSubMenu := [
        subMenu := Menu new.
        label size > 0 ifTrue: [ subMenu title: label ].
        self addAditionalSmalltalkToolItems: action to: subMenu.
        aMenu appendSubMenu: subMenu
    ].

    tools do: [ :item |
        item isNil
        ifTrue: [ aMenu appendSeparator ]
        ifFalse: [
            label := item at: 1.
            action := item at: 2.
            commandKey := item at: 3.
            (action isNil or: [label size = 0])
            ifTrue: [ aMenu appendSeparator ] "empty label will generate menu separators..."
            ifFalse: [
                action isArray "action array can be used for subMenus"
                ifTrue: [ appendSubMenu value    ]
                ifFalse: [ "other entries must standard menu settings (label action commandKey)"
                    aMenu
                        appendItem: label
                        action: action
                        commandKey: commandKey
                ]
            ]
        ]
    ]!
initializeToolItems
    "Private - Initialize default tool items."

    | toolItems |
    #addedByOStyle.
    toolItems := OrderedCollection new.
    toolItems
        add: nil;
        add: (Array with: '&Unused Memory...' with: #unusedMemory with: nil);
        add: (Array with: '&Purge Resources' with: #purgeResources with: nil);
        add: (Array with: '&Compress Changes' with: #compressChanges with: nil);
        add: nil;
        add: (Array with: 'C&lean Up' with: #cleanUp with: nil);
        add: (Array with: '&Find' with: (
            Array
                with: (Array with: '&Senders' with: #sendersInText with: nil)
                with: (Array with: '&Implementors' with: #implementorsInText with: nil)
                with: (Array with: '&Class' with: #findClassInText with: nil)
            ) with: nil);
        yourself.
    SmalltalkWindowPolicy toolItems: toolItems.! !

! Debugger methods !
walkback: anInteger
        "Private - select the walkback line at index position anInteger."
    | process sourceInterval start end oldMethod oldActivation processIndex |
    process := self debuggedProcess.
    walkbackIndex := anInteger.
    processIndex := self  processWalkbackIndex.
    oldActivation := self activation.
    oldMethod := self method.
    browseWalkback
        ifTrue: [
            ( process isBlockFrame: processIndex )
                ifTrue: [
                    method := process contextFor: processIndex.
                    method isBlockClosure ifFalse: [ method := process methodAt: processIndex ] ]
                ifFalse: [ method := process methodAt: processIndex ] ]
        ifFalse:
            [ oldMethod := nil.
            method := breakpoints at: walkbackIndex.
            method := method classField
                compiledMethodAt: method selector.
            ].
    ((oldActivation == method) and: [(self methodPane modified not)])
        ifFalse: [self updateMethodText].
    instIndex := nil.
    self changed: #tempList:.
    self changed: #instance:.
    start := end := 1.
    browseWalkback ifTrue: [
        object := process receiverAt: processIndex.
        self
            withInfoFor: self method
            do:
                [:info |
                sourceInterval := info sourceIntervalForProcessIC: self ic.
                start := sourceInterval first.
                end :=sourceInterval last]
              ifAbsent: []].
    self methodPane
		showSelectionFrom: start to: end;
		forceSelectionOntoDisplay. #modifiedByOStyle.
    self checkMenus.! !